package com.xubx.springboot_01demo.Task;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.entity.pojo.TopK;
import com.xubx.springboot_01demo.entity.vo.HotBlog;
import com.xubx.springboot_01demo.mapper.BlogLikeMapper;
import com.xubx.springboot_01demo.mapper.BlogsMapper;
import com.xubx.springboot_01demo.mapper.UserFavoriteBlogMapper;
import com.xubx.springboot_01demo.entity.pojo.BlogLike;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.pojo.UserFavoriteBlog;
import com.xubx.springboot_01demo.service.BlogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.nio.charset.StandardCharsets;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Component
@Slf4j
public class DataSync {
    @Resource
    private RedisTemplate redisTemplate;
    @Resource
    private BlogsMapper blogsMapper;
    @Resource
    private BlogLikeMapper blogLikeMapper;
    @Resource
    private UserFavoriteBlogMapper userFavoriteBlogMapper;
    @Autowired
    private BlogService blogService;
    @Autowired
    private ObjectMapper objectMapper;


    /**
     * 博客浏览数的数据同步 ，每小时执行一次
     */
//    @Scheduled(cron = "0 0 * * * ?")
    @Scheduled(cron = "0 * * * * ?")
    public void viewCountSync() {
        log.info("博客浏览数的数据同步定时任务执行");

        String lockKey = constantConfiguration.BLOG_VIEW_COUNT_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));
        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> viewCountKeys = redisTemplate.keys(constantConfiguration.BLOG_VIEW_COUNT + "*");
                if (viewCountKeys != null) {
                    for (String key : viewCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String expirekey = constantConfiguration.BLOG_VIEW_COUNT + blogIdStr + ":expired";

                        Boolean isExpired = Boolean.TRUE.equals(redisTemplate.hasKey(expirekey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(expirekey));
                        // 如果标签存在 则进行数据的更新，若不存在则表示数据已经同步 或者数据还没被更新
                        if (Boolean.TRUE.equals(isExpired)) {
                            String viewCount = (String) redisTemplate.opsForValue().get(key);
                            if (viewCount != null) {
                                //更新数据
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr)
                                        .set(Blogs::getViewCount, viewCount);
                                blogsMapper.update(null, updateWrapper);
                                // 删除逻辑过期标志
                                redisTemplate.delete(expirekey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客浏览数的数据同步任务无法执行");
        }
    }

    /**
     * 博客点赞数的数据同步定时任务执行
     */
    @Scheduled(cron = "0 * * * * ?")
    public void likeCountSync() {
        log.info("博客点赞数的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_LIKE_COUNT_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));

        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> likeCountKeys = redisTemplate.keys(constantConfiguration.BLOG_LIKE_COUNT + "*");
                if (likeCountKeys != null) {
                    likeCountKeys = likeCountKeys.stream().filter(key -> !key.endsWith(":expired")).collect(Collectors.toSet());
                    for (String key : likeCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String likeCountExpireKey = constantConfiguration.BLOG_LIKE_COUNT + blogIdStr + ":expired";

                        Boolean countExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(likeCountExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(likeCountExpireKey));

                        if (Boolean.TRUE.equals(countExpireFlag)) {
                            String likeCount = (String) redisTemplate.opsForValue().get(key);
                            if (likeCount != null) {
                                // 更新点赞次数到数据库
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr).set(Blogs::getLikeCount, Integer.parseInt(likeCount));
                                blogsMapper.update(null, updateWrapper);

                                // 删除逻辑过期标志
                                redisTemplate.delete(likeCountExpireKey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客点赞数的数据同步任务无法执行");
        }
    }

    /**
     * 博客点赞用户Hash集合的增量数据同步定时任务执行
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void likeUserSync() {
        log.info("博客点赞用户Hash集合的增量数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_LIKE_USERHASH_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));

        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> likeUserKeys = redisTemplate.keys(constantConfiguration.BLOG_LIKE_HASH + "*");
                if (likeUserKeys != null) {
                    likeUserKeys = likeUserKeys.stream().filter(key -> !key.endsWith(":expired")).collect(Collectors.toSet());
                    for (String key : likeUserKeys) {
                        String blogIdStr = key.split(":")[2];
                        String likeUserExpireKey = constantConfiguration.BLOG_LIKE_HASH + blogIdStr + ":expired";

                        Boolean userHashExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(likeUserExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(likeUserExpireKey));

                        if (Boolean.TRUE.equals(userHashExpireFlag)) {
                            // 获取博客点赞用户集合中的所有用户ID
                            Set<String> userIds = redisTemplate.opsForHash().keys(constantConfiguration.BLOG_LIKE_HASH + blogIdStr);
                            if (userIds != null) {
                                for (String userId : userIds) {
                                    String likeStatus = (String) redisTemplate.opsForHash().get(constantConfiguration.BLOG_LIKE_HASH + blogIdStr, userId);
                                    if ("true".equals(likeStatus)) {
                                        // 插入数据
                                        LambdaQueryWrapper<BlogLike> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(BlogLike::getBlogId, blogIdStr).eq(BlogLike::getUserId, userId);
                                        BlogLike blogLike = blogLikeMapper.selectOne(queryWrapper);

                                        if (blogLike == null) {
                                            BlogLike newBlogLike = new BlogLike();
                                            newBlogLike.setBlogId(Long.valueOf(blogIdStr));
                                            newBlogLike.setUserId(Long.valueOf(userId));
                                            newBlogLike.setCreatedTime(Timestamp.valueOf(LocalDateTime.now()));
                                            blogLikeMapper.addData(newBlogLike);
                                        }
                                    } else if ("false".equals(likeStatus)) {
                                        // 用户取消点赞 则删除记录
                                        LambdaQueryWrapper<BlogLike> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(BlogLike::getBlogId, blogIdStr).eq(BlogLike::getUserId, userId);
                                        blogLikeMapper.delete(queryWrapper);
                                    }
                                }
                                // 删除逻辑过期标志和哈希数据
                                redisTemplate.delete(likeUserExpireKey);
                                redisTemplate.delete(constantConfiguration.BLOG_LIKE_HASH + blogIdStr);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客点赞用户Hash集合的增量数据同步任务无法执行");
        }
    }

    /**
     * 博客收藏数的数据同步定时任务
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次，需要隔开，不能同一时间全部执行
    public void collectCountSync() {
        log.info("博客收藏数的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_COLLECT_COUNT_SYNC_LOCK;
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, "LOCKED", 30, TimeUnit.MINUTES);

        if (Boolean.TRUE.equals(lockAcquired)) {
            try {
                Set<String> collectCountKeys = redisTemplate.keys(constantConfiguration.BLOG_COLLECT_COUNT + "*");
                if (collectCountKeys != null) {
                    collectCountKeys = collectCountKeys.stream().filter(key -> !key.endsWith(":expired")).collect(Collectors.toSet());
                    for (String key : collectCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String collectCountExpireKey = constantConfiguration.BLOG_COLLECT_COUNT + blogIdStr + ":expired";

                        Boolean countExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(collectCountExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(collectCountExpireKey));

                        if (Boolean.TRUE.equals(countExpireFlag)) {
                            String collectCount = (String) redisTemplate.opsForValue().get(key);
                            if (collectCount != null) {
                                // 更新收藏次数到数据库
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr).set(Blogs::getCollectCount, Integer.parseInt(collectCount));
                                blogsMapper.update(null, updateWrapper);

                                // 删除逻辑过期标志
                                redisTemplate.delete(collectCountExpireKey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，跳过此次收藏数同步任务");
        }
    }

    /**
     * 博客收藏增量数据同步
     *
     * @throws JsonProcessingException
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void collectUserSync() {
        log.info("博客收藏用户Hash集合的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_COLLECT_USERHASH_SYNC_LOCK;
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, "LOCKED", 30, TimeUnit.MINUTES);

        if (Boolean.TRUE.equals(lockAcquired)) {
            try {
                // TODO 应该是同时获取到了逻辑过期标志的Key，需要进行去除
                Set<String> collectUserKeys = redisTemplate.keys(constantConfiguration.BLOG_COLLECT_HASH + "*");
                if (collectUserKeys != null) {
                    collectUserKeys = collectUserKeys.stream().filter(key -> !key.contains(":expired")).collect(Collectors.toSet());
                    for (String key : collectUserKeys) {
                        String favoriteIdStr = key.split(":")[2];
                        String collectUserExpireKey = constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr + ":expired";

                        Boolean userExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(collectUserExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(collectUserExpireKey));
                        if (Boolean.TRUE.equals(userExpireFlag)) {
                            // 获取博客收藏用户集合中的所有博客
                            Set<String> blogSet = redisTemplate.opsForHash().keys(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr);
                            if (blogSet != null) {
                                for (String blogIdStr : blogSet) {
                                    // 根据用户的收藏状态，更新数据
                                    String collectStatus = (String) redisTemplate.opsForHash().get(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr, blogIdStr);
                                    if ("true".equals(collectStatus)) {
                                        // 如果数据库中没有该数据则插入数据
                                        LambdaQueryWrapper<UserFavoriteBlog> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(UserFavoriteBlog::getBlogId, blogIdStr).eq(UserFavoriteBlog::getFavoriteId, favoriteIdStr);
                                        UserFavoriteBlog userFavoriteBlog = userFavoriteBlogMapper.selectOne(queryWrapper);
                                        if (userFavoriteBlog == null) {
                                            userFavoriteBlog = new UserFavoriteBlog();
                                            userFavoriteBlog.setBlogId(Integer.valueOf(blogIdStr));
                                            userFavoriteBlog.setFavoriteId(Integer.valueOf(favoriteIdStr));
                                            userFavoriteBlogMapper.insert(userFavoriteBlog);
                                        }
                                    } else if ("false".equals(collectStatus)) {
                                        // 用户取消收藏 删除数据
                                        LambdaQueryWrapper<UserFavoriteBlog> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(UserFavoriteBlog::getBlogId, blogIdStr).eq(UserFavoriteBlog::getFavoriteId, favoriteIdStr);
                                        userFavoriteBlogMapper.delete(queryWrapper);
                                    }
                                }
                                // 删除逻辑过期标志和Hash集合
                                redisTemplate.delete(collectUserExpireKey);
                                redisTemplate.delete(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr);
                            }

                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，跳过此次收藏用户Hash集合的数据同步任务");
        }
    }

    /**
     * 热度排行榜，全表分片扫描
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void hotRank() {
        log.info("热度排行榜刷新定时任务执行");

        final TopK topK = new TopK(10);
        long limit = 1000;
        long id = 0;

        List<Blogs> blogs = blogService.list(new LambdaQueryWrapper<Blogs>()
                .select(Blogs::getId, Blogs::getTitle, Blogs::getLikeCount, Blogs::getViewCount, Blogs::getCollectCount, Blogs::getCommentCount,
                        Blogs::getShareCount, Blogs::getCreatedTime).gt(Blogs::getId, id)
                .eq(Blogs::getStatus, "公开")
                .last("limit " + limit));

        while (!ObjectUtils.isEmpty(blogs)) {
            for (Blogs blog : blogs) {
                Long likeCount = blog.getLikeCount();
                Double viewCount = blog.getViewCount() * 0.8;
                Double collectCount = blog.getCollectCount() * 1.5;
                Long commentCount = blog.getCommentCount();
                Long shareCount = blog.getShareCount();
                final Date date = new Date();
                long decayTime = date.getTime() - blog.getCreatedTime().getTime();
                // 防止出现热度完全相同的情况，加入0.1-1.0之间的随机小数
                final double v = Math.random() * 0.9 + 0.1;
                final double hot = hot(likeCount + viewCount + collectCount + commentCount + shareCount + v, TimeUnit.MILLISECONDS.toDays(decayTime));
                final HotBlog hotBlog = new HotBlog(blog.getId(), hot, blog.getTitle());

                // add过程中，已存入热度最大的十个博客
                topK.add(hotBlog);
            }
            id = blogs.get(blogs.size() - 1).getId();
            blogs = blogService.list(new LambdaQueryWrapper<Blogs>().gt(Blogs::getId, id).eq(Blogs::getStatus, "公开").last("limit " + limit));
        }
        // 通过redis管道进行数据更新
        final byte[] key = constantConfiguration.HOT_RANK.getBytes();
        final List<HotBlog> hotBlogs = topK.get();// 已从大到小排序
        final Double minHot = hotBlogs.get(hotBlogs.size() - 1).getHot();
        redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            for (HotBlog hotBlog : hotBlogs) {
                final Double hot = hotBlog.getHot();
                // 由于只有十条数据，直接存实体类
                try {
                    hotBlog.setHot(null);
                    connection.zAdd(key, hot, objectMapper.writeValueAsString(hotBlog).getBytes());
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }
            return null;
        });

        // 删除热度低于minHot的元素
        redisTemplate.opsForZSet().removeRangeByScore(constantConfiguration.HOT_RANK, minHot, 0);
    }

    /**
     * 控制衰减速度的常数，a 值越大，时间对热度的影响越强，意味着热度衰减得越快
     */
    static double a = 0.011;

    /**
     * 热度半衰期公式
     */
    public static double hot(double weight, double t) {
        // Math.exp用于计算e的次方
        // 指数衰减函数，随着 t 增大，Math.exp(-a * t) 的值逐渐变小
        return weight * Math.exp(-a * t);
    }
}
