package com.xubx.springboot_01demo.WebSocket;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.xubx.springboot_01demo.entity.dto.SendMesDto;
import com.xubx.springboot_01demo.service.MessagesService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * WebSocket服务
 */
@Component
@ServerEndpoint("/ws/{username}")
@Slf4j
public class WebSocketServer {

    //存放会话对象
    private static Map<String, Session> sessionMap = new HashMap();
    static MessagesService messagesService;

    @Autowired
    public void setMessagesService(MessagesService messagesService) {
        WebSocketServer.messagesService = messagesService;
    }

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("username") String username) {
        System.out.println("客户端：" + username + "建立连接");
        sessionMap.put(username, session);
    }

    /**
     * @param message 客户端发送过来的消息
     * @Description: 收到消息触发事件，这个消息是连接人发送的消息
     * 收到客户端消息后调用的方法
     */
    @OnMessage
    public void onMessage(String message, @PathParam("username") String username) {
        ObjectMapper mapper = new ObjectMapper();
        System.out.println("收到来自客户端：" + username + "的信息:" + message);
        try {
            if (message.equals("closeWebsocket")) {
                sessionMap.remove(username);
                log.info("客户端：" + username + "断开连接");
                return;
            }
            SendMesDto sendMesDto = mapper.readValue(message, SendMesDto.class);
            sendMessageTo(sendMesDto);
            messagesService.sendMessages(sendMesDto);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 连接关闭调用的方法
     *
     * @param sid
     */
//    @OnClose
//    public void onClose(@PathParam("sid") String sid) {
//        System.out.println("连接断开:" + sid);
//        sessionMap.remove(sid);
//    }

    /**
     * 群发
     *
     * @param message
     */
    public void sendToAllClient(String message) {
        Collection<Session> sessions = sessionMap.values();
        for (Session session : sessions) {
            try {
                //服务器向客户端发送消息
                session.getBasicRemote().sendText(message);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void sendMessageTo(SendMesDto sendMesDto) {
        Session session = sessionMap.get(sendMesDto.getRecipient());
        try {
            session.getBasicRemote().sendText(new ObjectMapper().writeValueAsString(sendMesDto));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
