package com.xubx.springboot_01demo.controller;

import com.xubx.springboot_01demo.entity.dto.RelationshipDto;
import com.xubx.springboot_01demo.service.RelationshipService;
import com.xubx.springboot_01demo.utils.api.Result;
import com.xubx.springboot_01demo.utils.token.RequestHolder;
import com.xubx.springboot_01demo.entity.vo.UserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController //注解标识这是一个控制器类
@CrossOrigin    //加上CrossOrigin可解决跨域问题
@Api(tags = "用户关系接口")
@RequestMapping("/relationship")
public class RelationshipController {

    @Resource
    RelationshipService relationshipService;

    /**
     * 查询用户
     *
     * @param username
     * @return
     */
    @ApiOperation("查询用户")
    @GetMapping("/queryUser")
    public List<UserVo> queryUser(@RequestParam("username") String username) {
        return relationshipService.queryUser(username);
    }

    /**
     * 发送添加请求
     *
     * @param username
     * @return
     */
    @ApiOperation("发送添加请求")
    @GetMapping("/addRequest")
    public ResponseEntity<Result<Void>> requestFriend(@RequestParam("username") String username){
        String userId = RequestHolder.getuserId().toString();
        try {
            relationshipService.requestFriend(userId, username);
            // 成功返回
            Result<Void> result = new Result<>();
            result.setCode(200);
            result.setMessage("请求已发送");
            return new ResponseEntity<>(result, HttpStatus.OK);
        } catch (Exception e) {
            // 处理错误返回
            Result<Void> result = new Result<>();
            result.setCode(400);
            result.setMessage("请求发送失败: " + e.getMessage());
            return new ResponseEntity<>(result, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * 接受添加请求
     *
     * @param relationshipDto
     * @return
     */
    @ApiOperation("接受添加请求")
    @PostMapping("/acceptRequest")
    public ResponseEntity<Result<Void>> acceptFriend(@RequestBody RelationshipDto relationshipDto) {
        try {
            relationshipService.acceptFriend(relationshipDto.getUsername(), relationshipDto.getFriend());
            // 成功返回
            Result<Void> result = new Result<>();
            result.setCode(200);
            result.setMessage("添加成功");
            return new ResponseEntity<>(result, HttpStatus.OK);
        } catch (Exception e) {
            // 处理错误返回
            Result<Void> result = new Result<>();
            result.setCode(400);
            result.setMessage("添加失败: " + e.getMessage());
            return new ResponseEntity<>(result, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * 拒绝添加请求
     *
     * @param relationshipDto
     * @return
     */
    @ApiOperation("拒绝添加请求")
    @PostMapping("/refuseRequest")
    public ResponseEntity<Result<Void>> refuseFriend(@RequestBody RelationshipDto relationshipDto) {
        try {
            relationshipService.refuseFriend(relationshipDto.getUsername(), relationshipDto.getFriend());
            // 成功返回
            Result<Void> result = new Result<>();
            result.setCode(200);
            result.setMessage("拒绝成功");
            return new ResponseEntity<>(result, HttpStatus.OK);
        } catch (Exception e) {
            // 处理错误返回
            Result<Void> result = new Result<>();
            result.setCode(400);
            result.setMessage("拒绝失败: " + e.getMessage());
            return new ResponseEntity<>(result, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * 删除好友
     *
     * @param relationshipDto
     */
//    @ApiOperation("删除好友")
//    @PostMapping("/removeFriend")
//    public void removeFriend(@RequestBody RelationshipDto relationshipDto) {
//        relationshipService.removeFriend(RequestHolder.getuserId(), relationshipDto.getFriend());
//    }
//
//    /**
//     * 获取好友列表
//     * @return
//     */
//    @ApiOperation("获取好友列表")
//    @GetMapping("/getFriends")
//    public List<UserListVo> getFriends() {
//        return relationshipService.getFriends(RequestHolder.getuserId());
//    }


}
