package com.xubx.springboot_01demo.controller;


import com.alibaba.fastjson.JSONObject;
import com.xubx.springboot_01demo.entity.dto.user.EditMaterialDto;
import com.xubx.springboot_01demo.entity.dto.user.NewFavoritesDto;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.pojo.User;
import com.xubx.springboot_01demo.entity.pojo.UserFavorite;
import com.xubx.springboot_01demo.service.UserService;
import com.xubx.springboot_01demo.utils.token.RequestHolder;
import com.xubx.springboot_01demo.entity.vo.getUserInfoVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

@RestController //注解标识这是一个控制器类
@CrossOrigin    //加上CrossOrigin可解决跨域问题
@Slf4j
@Api(tags = "用户接口")
@RequestMapping("/user")
public class UserController {
    @Resource
    UserService userService;
    //    注入session

    /**
     * 用户注册
     *
     * @param user
     * @return
     */
    @PostMapping("/register")
    @ApiOperation("用户注册")
    public ResponseEntity<String> register(@RequestBody User user) {
        // 输入验证
        if (user.getUsername() == null || user.getUsername().isEmpty()) {
            return ResponseEntity.badRequest().body("用户名不能为空");
        }
        if (user.getPassword() == null || user.getPassword().isEmpty()) {
            return ResponseEntity.badRequest().body("密码不能为空");
        }
        try {
            if (userService.insertUser(user)) {
                return ResponseEntity.status(HttpStatus.CREATED).body("注册成功");
            } else {
                return ResponseEntity.status(HttpStatus.CONFLICT).body("用户已经存在");
            }
        } catch (Exception e) {
            // 异常处理
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("注册失败，服务器内部错误");
        }
    }

    /**
     * 用户登录
     *
     * @param user
     * @return
     */
    @PostMapping("/login")
    @ApiOperation("用户登录")
    public ResponseEntity<String> login(@RequestBody User user) {
        log.info("用户登录:{}", user);

        // 调用 Service 中的登录方法
        String token = userService.login(user);

        if (token != null) {
            return ResponseEntity.ok(token);
        } else {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("登录失败，用户名或密码错误");
        }
    }

    /**
     * 获取用户名
     *
     * @return
     */
    @GetMapping("/getUsername")
    @ApiOperation("获取当前用户名")
    public String getUserName() {
        return userService.getUser("").getUsername();
    }

    /**
     * 退出登录
     *
     * @return
     */
//    @RequestMapping("/uploadAvatar")
//    @ApiOperation("上传头像")
//    public void uploadAvatar(MultipartFile file) throws IOException {
//        String pType = file.getContentType();
//        pType = pType.substring(pType.indexOf("/") + 1);
//        if ("jpeg".equals(pType)) {
//            pType = "jpg";
//        }
//        long time = System.currentTimeMillis();
//        String currentWorkingDirectory = System.getProperty("user.dir");
//        String relativePath = "/images/avatar/" + time + "." + pType;
//        String absolutePath = currentWorkingDirectory + "/static" + relativePath;
//        file.transferTo(new File(absolutePath));
//        System.out.println("导入数据库的路径：" + relativePath + "当前用户Id：" + RequestHolder.getuserId());
//        userService.addAvatar(relativePath, RequestHolder.getuserId());
//    }

    /**
     * 获取头像
     *
     * @return
     */
    @GetMapping("/getAvatar")
    @ApiOperation("获取头像")
    public String getAvatar() {
        return userService.getUser("").getAvatar();
    }

    /**
     * 修改用户名
     *
     * @param username
     * @return
     */
    @GetMapping("usernameChange")
    @ApiOperation("修改用户名")
    public ResponseEntity<String> usernameChange(@RequestParam("username") String username) {
        log.info("修改用户名:{}", username);

        try {
            userService.usernameChange(username);
            return ResponseEntity.ok("修改成功！");
        } catch (Exception e) {
            return ResponseEntity.ok("修改失败！");
        }
    }

    /**
     * 修改密码
     *
     * @param oldPassword
     * @param newPassword
     * @return
     */
//    @GetMapping("passwordChange")
//    @ApiOperation("修改密码")
//    public ResponseEntity<String> passwordChange(@RequestParam("oldPassword") String oldPassword, @RequestParam("newPassword") String newPassword) {
//        if (userService.passwordChange(RequestHolder.getuserId(), oldPassword, newPassword)) {
//            return ResponseEntity.ok("修改成功！");
//        }
//        return ResponseEntity.ok("原密码输入错误！");
//    }
//
//    /**
//     * 查看是否有好友请求
//     *
//     * @return
//     */
//    @GetMapping("/checkFriendRequest")
//    public JSONObject haveFriendRequest() {
//        return userService.checkFriendRequest(RequestHolder.getuserId());
//    }

    /**
     * 获取用户信息
     *
     * @return {@link ResponseEntity }<{@link getUserInfoVo }>
     */
    @GetMapping("/getUserInfo")
    @ApiOperation("获取用户信息")
    public ResponseEntity<getUserInfoVo> getUserInfo(@RequestParam("userId") String userId) {
        log.info("获取用户信息:{}", userId);

        try {
            return ResponseEntity.ok(userService.getUserInfo(userId));
        } catch (Exception e) {
            log.info("获取用户信息失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
    }

    /**
     * 修改用户资料
     * @param editMaterialDto
     * @return {@link ResponseEntity }<{@link String }>
     */
    @PostMapping("/editMaterial")
    @ApiOperation("修改用户资料")
    public ResponseEntity<String> editMaterial(@RequestBody EditMaterialDto editMaterialDto) {
        log.info("修改用户资料");
        try {
            userService.editMaterial(editMaterialDto);
            return ResponseEntity.ok("修改成功！");
        } catch (Exception e) {
            return ResponseEntity.ok("修改失败！");
        }
    }

    /**
     * 新建收藏夹
     *
     * @param newFavorites
     * @return {@link ResponseEntity }<{@link String }>
     */
    @PostMapping("/newFavorites")
    @ApiOperation("新建收藏夹")
    public ResponseEntity<String> newFavorites(@Valid @RequestBody NewFavoritesDto newFavorites, BindingResult bindingResult) {
        log.info("新建收藏夹，入参：{}", JSONObject.toJSONString(newFavorites));

        if (bindingResult.hasErrors()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(bindingResult.getFieldError().getDefaultMessage());
        }

        Integer userId = RequestHolder.getuserId();
        if (userId == null) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("用户未登录");
        }

        try {
            String result = userService.newFavorites(newFavorites);
            if (result != null) {
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(result);
            }
            return ResponseEntity.ok("收藏夹创建成功");
        } catch (Exception e) {
            log.error("新建收藏夹失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("新建收藏夹失败");
        }
    }

    /**
     * 获取用户收藏夹列表
     * @return {@link ResponseEntity }<{@link List }<{@link UserFavorite }>>
     */
    @GetMapping("/getFavorites")
    @ApiOperation("获取收藏夹列表")
    public ResponseEntity<List<UserFavorite>> getUserFavorites(@RequestParam("userId") String userId) {
        log.info("获取收藏夹列表");

        try {
            return ResponseEntity.ok(userService.getUserFavorites(userId));
        } catch (Exception e) {
            log.error("获取收藏夹列表失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
    }

    /**
     * 获取收藏夹内的博客
     * @param favoriteId
     * @return {@link ResponseEntity }<{@link List }<{@link Blogs }>>
     */
    @GetMapping("/getBlogsByFavoriteId")
    @ApiOperation("根据收藏夹id获取收藏夹内的博客")
    public ResponseEntity<?> getBlogsByFavoriteId(@RequestParam("favoriteId") String favoriteId) {
        log.info("根据收藏夹id获取收藏夹内的博客: {}", favoriteId);

        try {
            return ResponseEntity.ok(userService.getBlogsByFavoriteId(favoriteId));
        } catch (Exception e) {
            log.error("根据收藏夹id获取收藏夹内的博客失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("根据收藏夹id获取收藏夹内的博客失败");
        }
    }

    /**
     * 获取用户粉丝列表
     * @param userId
     * @return {@link ResponseEntity }<{@link ? }>
     */
    @GetMapping("/getUserFans")
    @ApiOperation("获取用户粉丝列表")
    public ResponseEntity<?> getUserFans(@RequestParam("userId") String userId) {
        log.info("获取用户粉丝列表: {}", userId);

        try {
            return ResponseEntity.ok(userService.getUserFans(userId));
        } catch (Exception e) {
            log.error("获取用户粉丝列表失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("获取用户粉丝列表失败");
        }
    }

    /**
     * 获取用户关注列表
     * @param userId
     * @return {@link ResponseEntity }<{@link ? }>
     */
    @GetMapping("/getUserFollowList")
    @ApiOperation("获取用户关注列表")
    public ResponseEntity<?> getUserFollowList(@RequestParam("userId") String userId) {
        log.info("获取用户关注列表: {}", userId);

        try {
            return ResponseEntity.ok(userService.getUserFollowList(userId));
        } catch (Exception e) {
            log.error("获取用户关注列表失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("获取用户关注列表失败");
        }
    }

    /**
     * 关注/取消关注用户
     * @param followedId
     * @return {@link ResponseEntity }<{@link ? }>
     */
    @GetMapping("/followUser")
    @ApiOperation("关注/取消关注用户")
    public ResponseEntity<?> followUser(@RequestParam("userId") String followedId) {
        log.info("关注/取消关注用户: {}", followedId);

        try {
            return ResponseEntity.ok(userService.followUser(followedId));
        } catch (Exception e) {
            log.error("关注/取消关注用户失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("关注/取消关注用户失败");
        }
    }

    /**
     * 用户订阅分类，初始化用户模型
     * @param categories
     * @return {@link ResponseEntity }<{@link ? }>
     */
    @PostMapping("/subscribe")
    @ApiOperation("订阅分类")
    public ResponseEntity<?> subscribe(@RequestBody List<String> categories) {
        log.info("订阅分类: {}", categories);

        try {
            userService.subscribe(categories);
            return ResponseEntity.ok("订阅分类成功");
        } catch (Exception e) {
            log.error("订阅分类失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("订阅分类失败");
        }
    }

}
