package com.xubx.springboot_01demo.entity.pojo;

import com.xubx.springboot_01demo.entity.vo.HotBlog;

import java.util.*;

/**
 * @author Xubx
 * @date 2024/12/02
 */
public class TopK {
    private int k = 0;
    private PriorityQueue<HotBlog> queue;

    public TopK(int k) {
        this.k = k;
        // 使用一个最大堆，根据热度从大到小排序
        this.queue = new PriorityQueue<>((hv1, hv2) -> Double.compare(hv1.getHot(), hv2.getHot()));
    }

    /**
     * 向TopK队列中添加一个HotVideo对象
     * @param hotBlog 热度视频
     */
    public void add(HotBlog hotBlog) {
        if (queue.size() < k) {
            queue.add(hotBlog);
        } else if (queue.peek().getHot() < hotBlog.getHot()) {
            queue.poll();  // 移除最小的热度
            queue.add(hotBlog);  // 添加新的HotVideo
        }
    }

    /**
     * 获取排序后的TopK视频列表
     * @return 按热度降序排列的视频列表
     */
    public List<HotBlog> get() {
        // 转换为List并按热度降序排序
        List<HotBlog> list = new ArrayList<>(queue);
        // 由于PriorityQueue是小根堆，需要手动倒序排列
        Collections.reverse(list);
        return list;
    }
}
