package com.xubx.springboot_01demo.entity.vo;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.ToString;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Objects;

/**
 * @author Xubx
 * @date 2024/12/02
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@ToString
public class HotBlog implements Serializable {
    private static final long serialVersionUID = 1L;

    private String hotFormat;   // 格式化后的热度字符串（例如：500.0万）
    private Double hot;         // 视频的热度数值
    private Integer blogId;       // 视频ID
    private String title;       // 视频标题

    public HotBlog(Integer blogId, Double hot, String title){
        this.blogId = blogId;
        this.hot = hot;
        this.title = title;
    }

    /**
     * 格式化热度值为 "X万" 形式，例如 5000000 -> "500.0万"
     */
    public void formatHot() {
        if (hot != null) {
            // 使用 BigDecimal 处理精度问题，避免浮动误差
            BigDecimal bigDecimal = new BigDecimal(hot);
            BigDecimal decimal = bigDecimal.divide(new BigDecimal("10000"), 1, RoundingMode.HALF_UP); // 除以10000并保留一位小数
            this.hotFormat = String.format("%.1f万", decimal.doubleValue());
        }
    }

    /**
     * 重写 equals() 方法，根据 videoId 和 title 判断对象是否相等
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        HotBlog hotBlog = (HotBlog) o;
        return Objects.equals(blogId, hotBlog.blogId) && Objects.equals(title, hotBlog.title);
    }

    /**
     * 重写 hashCode() 方法，根据 videoId 和 title 生成唯一的哈希值
     */
    @Override
    public int hashCode() {
        return Objects.hash(blogId, title);
    }
}
