package com.xubx.springboot_01demo.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.xubx.springboot_01demo.entity.dto.blog.AddBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.CollectBlogDto;
import com.xubx.springboot_01demo.entity.pojo.Blogs;

import java.util.List;

public interface BlogsMapper extends BaseMapper<Blogs> {
    //获取所有博客文章信息
    List<Blogs> findAllBlogs();
    //根据id获取博客
    Blogs findByIdBlogs(int id);
    //新增博客
    void addBlogs(AddBlogDto blogs);
    //更新博客
    void updateBlogs(Blogs blogs);
    //删除博客
    void deleteBlogs(int id);

    /**
     * 插入博客标签中间表
     * @param blogId
     * @param tagId
     */
    void insertTags(int blogId, String tagId);

    /**
     * 获取该用户总访问量
     * @param userId
     * @return {@link Integer }
     */
    String getTotalViewCount(Integer userId);

    /**
     * 获取该用户总博客数
     * @param userId
     * @return {@link Integer }
     */
    String getTotalBlogCount(Integer userId);

    /**
     * 获取该用户被收藏总数
     * @param userId
     * @return {@link Integer }
     */
    String getTotalCollectCount(Integer userId);

    /**
     * 获取该用户被点赞总数
     * @param userId
     * @return {@link Integer }
     */
    String getTotalLikeCount(Integer userId);

    /**
     * 获取该用户被评论总数
     * @param userId
     * @return {@link Integer }
     */
    String getTotalCommentCount(Integer userId);

    /**
     * 获取该用户被分享总数
     * @param userId
     * @return {@link Integer }
     */
    String getTotalShareCount(Integer userId);

    /**
     * 获取某个博客的该用户的点赞状态
     * @param blogId
     * @param userId
     * @return {@link Integer }
     */
    Integer getLikeStatus(int blogId, Integer userId);

    /**
     * 获取某个博客的该用户的收藏状态
     * @param blogId
     * @param userId
     * @return {@link Integer }
     */
    Integer getCollectStatus(int blogId, Integer userId);

    /**
     * 获取某个博客的点赞数
     * @param blogId
     * @return {@link Integer }
     */
    String getLikeCount(int blogId);

    /**
     * 获取某个博客的浏览数
     * @param blogId
     * @return {@link Object }
     */
    String getViewCount(int blogId);

    /**
     * 获取某个博客的收藏数
     * @param blogId
     * @return {@link Object }
     */
    String getCollectCount(int blogId);

    /**
     * 获取某个博客的评论数
     * @param blogId
     * @return {@link Object }
     */
    String getCommentCount(int blogId);

    /**
     * 当前用户进行博客的收藏
     * @param collectBlogDto
     */
    void collectBlog(CollectBlogDto collectBlogDto);

    List<Blogs> getBlogsByFavoriteId(String favoriteId);

    /**
     * 获取该博客的所有点赞用户id
     * @param blogId
     * @return {@link List }<{@link String }>
     */
    List<String> getUserIdsByBlogId(String blogId);

    /**
     * 获取博客下的所有标签
     * @param blogId
     * @return {@link List }<{@link String }>
     */
    List<String> getTagsByBlogId(Integer blogId);
}
