package com.xubx.springboot_01demo.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.xubx.springboot_01demo.entity.dto.blog.AddBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.CollectBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.GetUserBlogsDto;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.vo.GetBlogDetailVo;
import com.xubx.springboot_01demo.entity.vo.HotBlog;
import org.springframework.http.ResponseEntity;

import java.util.List;

public interface BlogService extends IService<Blogs> {
    //获取所有博客
    List<Blogs> findAllBlogs();

    //根据id获取博客
    GetBlogDetailVo findByIdBlogs(int blogId);
    //添加博客
    void addBlogs(AddBlogDto blogs);
    //更新博客
    void updateBlogs(Blogs blogs);
    //删除博客
    void deleteBlogs(int id);

    /**
     * 查看某个用户的所有博客
     * @param getUserBlogsDto
     * @return {@link Page }<{@link Blogs }>
     */
    Page<Blogs> findBlogsByUserId(GetUserBlogsDto getUserBlogsDto);

    /**
     * 浏览博客
     * @param blogId
     */
    void viewBlog(int blogId);

    /**
     * 点赞博客
     * @param blogId
     */
    void likeBlog(int blogId);

    /**
     * 收藏博客
     * @param collectBlogDto
     * @throws JsonProcessingException
     */
    void collectBlog(CollectBlogDto collectBlogDto) throws JsonProcessingException;

    /**
     * 主页兴趣推送 -> 推荐
     * @return {@link List }<{@link Blogs }>
     */
    List<Blogs> pushBlogs();

    /**
     * 热度排行榜
     * @return {@link List }<{@link HotBlog }>
     */
    List<HotBlog> listHotRank();

    /**
     * 分类推送
     * @param categoryId
     * @return {@link List }<{@link Blogs }>
     */
    List<Blogs> pushBlogByCategory(Integer categoryId);
}
