package com.xubx.springboot_01demo.service;

import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.pojo.User;

import java.util.Collection;
import java.util.List;

/**
 * 兴趣推送
 * @author Xubx
 * @date 2024/11/22
 */
public interface InterestPushService {

    /**
     * 初始化用户模型 --> 订阅分类
     * @param userId
     * @param tags
     */
    void initUserModel(Integer userId, List<String> tags);

    /**
     * 用户模型修改概率 : 可分批次发送
     * 修改场景:
     * 1.观看浏览量到达总时长1/5  +0.5概率
     * 2.观看浏览量未到总时长1/5 -0.5概率
     * 3.点赞视频  +1概率
     * 4.收藏视频  +2概率
     */
    void updateUserModel(Integer userId, Integer blogId, Double score);

    /**
     *根据用户模型推送博客 -> 兴趣推送
     * @param user
     * @return {@link List }<{@link String }>
     */
    Collection<String> listBlogsByUserModel(User user);

    /**
     * 推入系统标签库
     *
     * @param bligId
     * @param tags
     */
    void pushTagsStockIn(Integer bligId, List<String> tags);

    /**
     * 推入分类库,用于后续随机推送分类视频
     *
     * @param blog
     */
    void pushCategoryStockIn(Blogs blog);

    /**
     * 分类推送，根据分类随机推送
     * @param categoryId
     * @return {@link Collection }<{@link String }>
     */
    Collection<String> listBlogsByCategoryId(Integer categoryId);

    /**
     * 删除分类库中的博客
     * @param blog
     */
    void deletCategoryStockIn(Blogs blog);

    /**
     * 删除标签库中的博客
     * @param blog
     */
    void deletTagsStockIn(Blogs blog);
}
