package com.xubx.springboot_01demo.service.impl;

import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.mapper.RelationshipMapper;
import com.xubx.springboot_01demo.entity.pojo.Relationship;
import com.xubx.springboot_01demo.entity.pojo.User;
import com.xubx.springboot_01demo.service.RelationshipService;
import com.xubx.springboot_01demo.service.UserService;
import com.xubx.springboot_01demo.entity.vo.UserListVo;
import com.xubx.springboot_01demo.entity.vo.UserVo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
@Service
public class RelationshipServiceImpl implements RelationshipService {

    @Resource
    RelationshipMapper relationshipMapper;
    @Resource
    UserService userService;


    /**
     * 根据用户名模糊查询用户
     *
     * @param username
     * @return
     */
    @Override
    public List<UserVo> queryUser(String username) {
        List<UserVo> userDtos = new ArrayList<>();

        //将User转换为UserDto
        for(User u : relationshipMapper.findUserByUsername(userService.getUser("").getUsername(),username)){
            UserVo userDto = new UserVo();
            //将User的属性复制到UserDto
            BeanUtils.copyProperties(u,userDto);
            userDtos.add(userDto);
        }
        return userDtos;
    }

    /**
     * 发送添加请求
     *
     * @param username
     * @param friendName
     */
    @Override
    public void requestFriend(String username, String friendName) {
        //设置status为pending
        Relationship relationship = new Relationship();
        relationship.setUsername(username);
        relationship.setFriend(friendName);
        relationship.setStatus(constantConfiguration.Add_Request);
        relationship.setCreated(new Timestamp(System.currentTimeMillis()));
        relationshipMapper.addFriendRequest(relationship);
    }

    /**
     * 接受添加请求
     *
     * @param username
     * @param friend
     */
    @Override
    public void acceptFriend(String username, String friend) {
        //将status改为accepted
        relationshipMapper.acceptFriendRequest(username, friend, constantConfiguration.Accept_Request);

    }

    /**
     * 拒绝添加请求
     *
     * @param username
     * @param friend
     */
    @Override
    public void refuseFriend(String username, String friend) {
        //直接删除
        relationshipMapper.removeFriend(username, friend);
    }

    /**
     * 删除好友
     *
     * @param username
     * @param friendName
     */
    @Override
    public void removeFriend(String username, String friendName) {
        relationshipMapper.removeFriend(username, friendName);
    }

    /**
     * 获取好友列表
     *
     * @param username
     * @return
     */
    @Override
    public List<UserListVo> getFriends(String username) {
        //通过用户名查找好友
        return relationshipMapper.getFriends(username);
    }
}
