package com.xubx.springboot_01demo.service.impl;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.entity.dto.user.EditMaterialDto;
import com.xubx.springboot_01demo.entity.dto.user.NewFavoritesDto;
import com.xubx.springboot_01demo.entity.dto.user.UserSubscription;
import com.xubx.springboot_01demo.entity.pojo.User;
import com.xubx.springboot_01demo.mapper.*;
import com.xubx.springboot_01demo.entity.pojo.*;
import com.xubx.springboot_01demo.service.InterestPushService;
import com.xubx.springboot_01demo.service.UserService;
import com.xubx.springboot_01demo.service.UserSubscriptionService;
import com.xubx.springboot_01demo.utils.token.RequestHolder;
import com.xubx.springboot_01demo.utils.token.TokenGenerate;
import com.xubx.springboot_01demo.entity.vo.UserModelVo;
import com.xubx.springboot_01demo.entity.vo.getUserInfoVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpSession;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

//Service进行具体的业务处理
@Service
public class UserServiceImpl implements UserService {
    @Resource
    private UserMapper userMapper;
    @Resource
    private RelationshipMapper relationshipMapper;
    @Resource
    private BlogsMapper blogMapper;
    @Resource
    private UserFavoriteMapper userFavoriteMapper;
    @Autowired
    private RedisTemplate redisTemplate;
    @Resource
    private HttpSession session;
    @Resource
    private UserFollowMapper userFollowMapper;
    @Autowired
    private UserSubscriptionService userSubscriptionService;
    @Autowired
    private InterestPushService interestPushService;

    //登陆，获取User对象
    public String login(User user) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getUsername, user.getUsername());
        User userByUsername = userMapper.selectOne(queryWrapper);
        RequestHolder.add(userByUsername.getId());

        // 校验用户是否存在以及密码是否匹配
        if (userByUsername != null && Objects.equals(userByUsername.getPassword(), user.getPassword())) {
            // 生成 Token
            String token = new TokenGenerate().generateToken(userByUsername.getId());
            // 将用户名存入 session 中
            session.setAttribute("username", userByUsername.getUsername());
            return token;
        }

        // 登录失败返回 null
        return null;
    }

    //注册，新增
    @Override
    public Boolean insertUser(User user) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getUsername, user.getUsername());

        // 检查用户名是否已存在
        if (userMapper.selectOne(queryWrapper) != null) {
            return false;
        }
        // 用户名不存在,插入新用户
        userMapper.insertUser(user);
        return true;
    }

    @Override
    public void addAvatar(String path, String userId) {
        LambdaUpdateWrapper<User> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(User::getId, userId)
                .set(User::getAvatar, path);
        userMapper.update(null, updateWrapper);
    }

    @Override
    public String getAvatar(String name) {
        return userMapper.getAvatar(name);
    }

    @Override
    public void usernameChange(String username) {
        LambdaUpdateWrapper<User> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(User::getId, RequestHolder.getuserId())
                .set(User::getUsername, username);
        userMapper.update(null, updateWrapper);
    }

    @Override
    public boolean passwordChange(String username, String oldPassword, String newPassword) {
        //检查输入的原密码是否正确
        if (oldPassword.equals(userMapper.getPasswordByname(username))) {
            userMapper.passwordChange(username, newPassword);
            return true;
        }
        return false;
    }

    //查看是否有好友请求
    @Override
    public JSONObject checkFriendRequest(String getuserId) {
        List<Relationship> relationships = relationshipMapper.checkFriendRequest(getuserId);
        JSONArray jsonArray = new JSONArray();
        for (Relationship relationship : relationships) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("id", relationship.getId());
            jsonObject.put("username", relationship.getUsername());
            jsonObject.put("friend", relationship.getFriend());
            jsonObject.put("created", relationship.getCreated().toString());
            jsonObject.put("initiator", relationship.getInitiator());
            jsonObject.put("avatar", userMapper.getAvatar(relationship.getInitiator()));
            jsonArray.add(jsonObject);
        }
        JSONObject result = new JSONObject();
        result.put("friendRequests", jsonArray);
        return result;
    }

    /**
     * 获取用户
     *
     * @param userId
     * @return {@link User }
     */
    @Override
    public User getUser(String userId) {
        User user;
        if (userId != null && !userId.equals("")) {
            user = userMapper.selectById(userId);
        } else {
            // 如果userId为空，则获取当前登录用户的userId
            user = userMapper.selectById(RequestHolder.getuserId());
        }
        return user;
    }

    /**
     * 获取用户信息
     *
     * @param userId
     * @return {@link getUserInfoVo }
     */
    @Override
    public getUserInfoVo getUserInfo(String userId) {
        User user = getUser(userId);
        final Integer finalUserId = user.getId();

        getUserInfoVo getUserInfoVo = new getUserInfoVo();
        getUserInfoVo.setUsername(user.getUsername());
        getUserInfoVo.setAvatar(user.getAvatar());
        getUserInfoVo.setBio(user.getBio());
        getUserInfoVo.setJoinBlogTime(user.getCreatedTime());
        getUserInfoVo.setTotalViewCount(getCachedValue(constantConfiguration.USER_TOTAL_VIEW_COUNT + finalUserId, () -> blogMapper.getTotalViewCount(finalUserId)));
        getUserInfoVo.setTotalBlogCount(getCachedValue(constantConfiguration.USER_TOTAL_BLOG_COUNT + finalUserId, () -> blogMapper.getTotalBlogCount(finalUserId)));
        getUserInfoVo.setTotalCollectCount(getCachedValue(constantConfiguration.USER_TOTAL_COLLECT_COUNT + finalUserId, () -> blogMapper.getTotalCollectCount(finalUserId)));
        getUserInfoVo.setTotalLikeCount(getCachedValue(constantConfiguration.USER_TOTAL_LIKE_COUNT + finalUserId, () -> blogMapper.getTotalLikeCount(finalUserId)));
        getUserInfoVo.setTotalCommentCount(getCachedValue(constantConfiguration.USER_TOTAL_COMMENT_COUNT + finalUserId, () -> blogMapper.getTotalCommentCount(finalUserId)));
        getUserInfoVo.setTotalShareCount(getCachedValue(constantConfiguration.USER_TOTAL_SHARE_COUNT + finalUserId, () -> blogMapper.getTotalShareCount(finalUserId)));
        getUserInfoVo.setTotalFollowerCount(getCachedValue(constantConfiguration.USER_TOTAL_FOLLOWER_COUNT + finalUserId, () -> relationshipMapper.getTotalFollowerCount(finalUserId)));
        return getUserInfoVo;
    }

    /**
     * 获取缓存值通用方法
     *
     * @param key
     * @param dbQuery
     * @return {@link Integer }
     */
    private String getCachedValue(String key, Supplier<String> dbQuery) {
        String value = (String) redisTemplate.opsForValue().get(key);
        if (value == null) {
            value = dbQuery.get();
            redisTemplate.opsForValue().set(key, value, 30, TimeUnit.MINUTES);
        }
        return value;
    }

    /**
     * 修改个人资料
     */
    @Override
    public void editMaterial(EditMaterialDto editMaterialDto) {
        LocalDateTime now = LocalDateTime.now();
        User user = new User();
        user.setId(RequestHolder.getuserId());
        user.setUsername(editMaterialDto.getUsername());
        user.setAvatar(editMaterialDto.getAvatar());
        user.setGender(editMaterialDto.getGender());
        user.setBio(editMaterialDto.getBio());
        user.setUpdateTime(Timestamp.valueOf(now));
        userMapper.updateById(user);
    }

    /**
     * 新建收藏夹
     *
     * @param newFavorites
     */
    @Override
    public String newFavorites(NewFavoritesDto newFavorites) {
        // 检查标题是否已存在
        LambdaQueryWrapper<UserFavorite> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(UserFavorite::getName, newFavorites.getName())
                .eq(UserFavorite::getUserId, RequestHolder.getuserId());
        int count = userFavoriteMapper.selectCount(wrapper);

        if (count > 0) {
            return "该收藏夹已存在";
        } else {
            UserFavorite userFavorite = new UserFavorite();
            userFavorite.setName(newFavorites.getName());
            userFavorite.setUserId(RequestHolder.getuserId());
            userFavorite.setDescription(newFavorites.getDescription());
            userFavorite.setCreatedTime(Timestamp.valueOf(LocalDateTime.now()));
            // 检查是否由收藏夹状态
            if (newFavorites.getFavoriteStatus() != null) {
                userFavorite.setFavoriteStatus("私密");
            } else {
                userFavorite.setFavoriteStatus("公开");
            }
            userFavoriteMapper.insert(userFavorite);
            return null; // 创建成功，无错误消息
        }

    }

    /**
     * 获取用户的收藏夹列表
     *
     * @param userId
     * @return {@link List }<{@link UserFavorite }>
     */
    @Override
    public List<UserFavorite> getUserFavorites(String userId) {
        Integer currentUserId = RequestHolder.getuserId();

        if (currentUserId == null) {
            throw new IllegalStateException("用户未登录");
        }

        LambdaQueryWrapper<UserFavorite> wrapper = new LambdaQueryWrapper<>();
        // 首先判断是否是获取当前用户的收藏夹列表
        if (userId == null || userId.isEmpty() || userId.equals(currentUserId.toString())) {
            // 获取当前用户的全部收藏夹
            // TODO 收藏夹中的博客数量不是实时的
            wrapper.eq(UserFavorite::getUserId, currentUserId)
                    .orderByDesc(UserFavorite::getCreatedTime);
            return userFavoriteMapper.selectList(wrapper);
        } else {
            // 获取指定用户的公开的收藏夹
            wrapper.eq(UserFavorite::getUserId, userId)
                    .eq(UserFavorite::getFavoriteStatus, "公开")
                    .orderByDesc(UserFavorite::getCreatedTime);
            return userFavoriteMapper.selectList(wrapper);
        }
    }

    /**
     * 获取收藏夹中的博客列表
     *
     * @param favoriteId
     * @return {@link List }<{@link Blogs }>
     */
    @Override
    public List<Blogs> getBlogsByFavoriteId(String favoriteId) {
        Integer userId = RequestHolder.getuserId();
        if (userId == null) {
            throw new IllegalStateException("用户未登录");
        }

        // 首先获取数据库中的数据
        List<Blogs> blogsFromDb = blogMapper.getBlogsByFavoriteId(favoriteId);

        // 然后获取缓存中的数据
        String userCollectBlogsHashKey = constantConfiguration.BLOG_COLLECT_HASH + favoriteId;
        if (redisTemplate.hasKey(userCollectBlogsHashKey)) {
            // 缓存中有数据，需要拼接到数据库获取到的数据
            Set<String> blogSet = redisTemplate.opsForHash().keys(userCollectBlogsHashKey);
            if (blogSet != null && !blogSet.isEmpty()) {
                for (String blogIdStr : blogSet) {
                    String collectStatus = (String) redisTemplate.opsForHash().get(userCollectBlogsHashKey, blogIdStr);
                    Blogs blog = blogMapper.findByIdBlogs(Integer.parseInt(blogIdStr));
                    if ("true".equals(collectStatus)) {
                        blogsFromDb.add(blog);
                    } else if ("false".equals(collectStatus)) {
                        blogsFromDb.remove(blog);
                    }
                }
            }
        }
        return blogsFromDb;
    }

    /**
     * 获取用户粉丝列表
     * @param userId
     * @return {@link List }<{@link User }>
     */
    @Override
    public List<User> getUserFans(String userId) {
        String currentUserId = String.valueOf(RequestHolder.getuserId());

        if (currentUserId == null) {
            throw new IllegalStateException("用户未登录");
        }
        // 判断是否是获取当前用户的粉丝列表
        if (userId == null || userId.isEmpty() || userId.equals(currentUserId)) {
            return userMapper.getUserFans(currentUserId);
        } else {
            return userMapper.getUserFans(userId);
        }
    }

    /**
     * 获取用户关注列表
     * @param userId
     * @return {@link List }<{@link User }>
     */
    @Override
    public List<User> getUserFollowList(String userId) {
        String currentUserId = String.valueOf(RequestHolder.getuserId());

        if (currentUserId == null) {
            throw new IllegalStateException("用户未登录");
        }
        // 判断是否是获取当前用户的粉丝列表
        if (userId == null || userId.isEmpty() || userId.equals(currentUserId)) {
            return userMapper.getUserFollowList(currentUserId);
        } else {
            return userMapper.getUserFollowList(userId);
        }
    }

    /**
     * 关注/取消关注用户
     * @param followedId
     * @return {@link Object }
     */
    @Override
    public Object followUser(String followedId) {
        Integer currentUserId = RequestHolder.getuserId();

        if (currentUserId == null) {
            throw new IllegalStateException("用户未登录");
        }
        if (followedId == null || followedId.isEmpty()) {
            throw new IllegalArgumentException("被关注用户id不能为空");
        }
        if (followedId.equals(currentUserId)) {
            throw new IllegalArgumentException("不能关注自己");
        }

        LambdaQueryWrapper<UserFollow> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserFollow::getFollowerId, currentUserId)
                    .eq(UserFollow::getFollowedId, followedId);
        // 判断是否已经关注
        if (userFollowMapper.selectOne(queryWrapper) != null) {
            // 已经关注，取消关注
            userFollowMapper.delete(queryWrapper);
            return "取消关注成功";
        } else {
            // 未关注，添加关注
            UserFollow userFollow = new UserFollow();
            userFollow.setFollowerId(currentUserId);
            userFollow.setFollowedId(Integer.valueOf(followedId));
            userFollow.setCreatedTime(new Timestamp(System.currentTimeMillis()));
            userFollowMapper.insert(userFollow);
            return "关注成功";
        }

    }

    /**
     * 用户订阅分类，初始化用户模型
     * @param categories
     * @return {@link Object }
     */
    @Override
    public void subscribe(List<String> categories) {
        Integer currentUserId = RequestHolder.getuserId();
        if (currentUserId == null) {
            throw new IllegalStateException("用户未登录");
        }
        if (categories == null || categories.isEmpty()) {
            throw new IllegalArgumentException("分类不能为空");
        }
        List<UserSubscription> userSubscriptions = new ArrayList<>();
        for (String category : categories) {
            UserSubscription userSubscription = new UserSubscription();
            userSubscription.setUserId(currentUserId);
            userSubscription.setCategoryId(Integer.valueOf(category));
            userSubscriptions.add(userSubscription);
        }
        userSubscriptionService.remove(new LambdaQueryWrapper<UserSubscription>().eq(UserSubscription::getUserId, currentUserId));
        userSubscriptionService.saveBatch(userSubscriptions);

        // 初始化模型
        UserModelVo userModelVo = new UserModelVo();
        userModelVo.setUserId(currentUserId);
        List<String> lables = new ArrayList<>();
        for (String category : categories) {
            lables.addAll(userSubscriptionService.gettagsByCategoryId(Integer.valueOf(category)));
        }
        userModelVo.setLableNames(lables);
        initModel(userModelVo);
    }

    public void initModel(UserModelVo userModelVo) {
        // 初始化模型
        interestPushService.initUserModel(userModelVo.getUserId(),userModelVo.getLableNames());
    }

}
